/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/* Komodo command handling.
   This file encapsulates some of the magic we do w.r.t. command handling.
   I'm not sure I don't want to get rid of some of this, but we'll see.

   There are two public functions:

        command_doCommand(command)

    and
        command_updateCommandset(commandset)

 */

var commandLog = null;
function command_onload() {
    commandLog = getLoggingMgr().getLogger('commands');
    //commandLog.setLevel(LOG_DEBUG);
}

// Returns "true" is the command is a "checkbox" type command - ie,
// if supported, the "isCommandEnabled()" function determines the check-state,
// rather than the grayness of the command itself.
function _commandNode_isCheckCommand(node)
{
    if (node) {
        //dump("Node: " + node + "\n");
        var t = node.getAttribute("type").toLowerCase();
        if (t == "checkbox" || t == "radio") {
            return true;
        } else {
            return node.hasAttribute("toggled");
        }
    }
    return false;
}

// XXX - Unused
//// Returns true if the command is enabled or checked.
//function _command_isCommandEnabled(id) {
//    var node = top.document.getElementById(id);
//    if (node) {
//        if (_commandNode_isCheckCommand(node)) {
//            return node.getAttribute("checked")
//        }
//        return !node.getAttribute("disabled")
//    }
//    commandLog.warn("isCommandEnabled - no ID '" + id + "'");
//    return false;
//}

// Returns true if the command was handled (even if it is disabled),
// or false if no handler can be found, or an error occurs.
function _command_updateCommand(command, commandNode, controller /* = null */) {
//    gTimeline.startTimer('    updating command: ' + command);
    try {
        var found = false;
        if (controller == undefined || controller == null) {
            controller = _command_getControllerForCommand(command);
        }

        var enabled = false;
        if (controller) {
            enabled = controller.isCommandEnabled(command);
            found = true;
        }
        commandLog.debug("Command: " + command + " is set to " + enabled)
        _command_setCommandEnabled(command, commandNode, found, enabled);
    } catch (e) {
        commandLog.exception(e,"doing _command_updateCommand: " + command + ': ');
    }
//    gTimeline.stopTimer('    updating command: ' + command);
//    gTimeline.markTimer('    updating command: ' + command);
    return found;
}

function command_updateCommandset(commandset) {
//    gTimeline.startTimer('updating commandset: ' + commandset.id);
    commandLog.info('command_updateCommandset: ' + commandset.id);
    var childNodes = commandset.childNodes;
    var length = childNodes.length;
    for (var i = 0; i < length; i++) {
        var commandID = childNodes[i].id;
        if (commandID) {
            if (!_command_updateCommand(commandID, childNodes[i])) {
                commandLog.debug("command_updateCommandset - " + commandset.id +
                          " - command ID '" + commandID + "' - no controller available!");
            }
        } else {
            commandLog.warn("command_updateCommandset - " + commandset.id + " - element no " + i + " has no ID!!");
        }
    }
//    gTimeline.stopTimer('updating commandset: ' + commandset.id);
//    gTimeline.markTimer('updating commandset: ' + commandset.id);
}

function _command_getControllerForCommand(command) {
    var commandDispatcher = top.document.commandDispatcher
    var focusedElement = commandDispatcher.focusedElement;
    var controller;

    // If html:embed tag, *must* go directly to its controller.
    // Can not use the commandDispatcher first, as some builtin commands
    // will always be satisfied, and never defer to the plugin.
    // this works in combination with our plugin events patch, which
    // must be present for the plugin to get key events in the first place
    if (focusedElement && focusedElement.parentNode &&
        (focusedElement.tagName == "html:embed")) {
        controller = focusedElement.parentNode.controllers.getControllerForCommand(command);
    }

    if (!controller)
        controller = commandDispatcher.getControllerForCommand(command);
    return controller
}


function _command_setCommandEnabled(id, node, supported, enabled)
{
    if ( ! node ) {
        node = top.document.getElementById(id);
    }
    if ( node ) {
        var really_enabled = supported;
        if (supported && _commandNode_isCheckCommand(node)) {
            if ( enabled ) {
                node.setAttribute('checked', 'true');
                node.setAttribute('toggled', 'true');
            }
            else {
                node.setAttribute('checked', 'false');
                node.setAttribute('toggled', 'false');
            }
        } else {
            really_enabled = enabled;
        }
        if (really_enabled)
            node.removeAttribute("disabled");
        else
            node.setAttribute('disabled', 'true');
    } else {
        commandLog.warn("Attempting to update command '" + id + "', but no node available");
    }
}

function koRequestUpdateCommands(event) {
    window.updateCommands(event);
    commandLog.warn("using old koRequestUpdateCommands: " + event);
}

// Execute a bit of JS code in a manner consistent with
// executing commands -- i.e. respect whether we're in macro
// mode or in "repeat next command" mode
function command_doCode(cmdId, code) {
    try {
        // gMacroMgr can be undefined if we're not in the main komodo.xul window (e.g. Rx commands)
        if (typeof(gMacroMgr) != 'undefined' &&
            gMacroMgr &&
            gMacroMgr.mode == 'recording') {
            gMacroMgr.appendCommand(cmdId);
        }
        var numRepeats = 1
        // if commands.js is included (e.g. in a dialog) but not isearch.js,
        // then gIsearchController is undefined -- we obviously can't be in
        // a 'repeated command' situation
        if (typeof(gIsearchController) != 'undefined' &&
            gIsearchController.inRepeatCounterAccumulation) {
            numRepeats = gIsearchController.repeatCounter;
            gCancelMultiHandler();
        }
        for (var i = 0; i < numRepeats; i++) {
            eval(code);
        }
    } catch (e) {
        commandLog.exception(e,"An error occurred executing the "+code+" code fragment");
        throw Components.results.NS_ERROR_FAILURE;
    }
}

function command_doCodeAsync(cmdId, code) {
    try {
        window.setTimeout("command_doCode('"+cmdId + '" , "' + code.replace('\\', '\\\\', 'g')+"')", 0);
    } catch (e) {
        commandLog.exception(e);
    }
}

function command_doCommandAsync(command, event) {
    // determine if this command is executed via a keybinding, if so, just
    // do the command synchrounously
    if (typeof(event) != 'undefined' && event.type == "keypress") {
        // for keybindings to work properly, we must cancel the keypress event
        // that caused this command to be issued.  This prevents a key command
        // getting executed twice.
        event.cancelBubble = true;
        event.preventDefault();
        return command_doCommand(command);
    }
    // otherwise, do this in a timeout, which prevents 'sticky' menu's and such
    try {
        window.setTimeout("command_doCommand('"+command.replace('\\', '\\\\', 'g')+"')", 0);
    } catch (e) {
        commandLog.exception(e);
    }
    return true;
}

// Execute a command.
function command_doCommand(command) {
    commandLog.debug("command_doCommand(" + command + ")");
    var rc = false;
    try {
        var controller = _command_getControllerForCommand(command);
        if (controller) {
            var commandNode = document.getElementById(command);
            // gMacroMgr can be undefined if we're not in the main komodo.xul window (e.g. Rx commands)
            if (typeof(gMacroMgr) != 'undefined' && gMacroMgr && gMacroMgr.mode == 'recording') {
                if (commandNode && !commandNode.hasAttribute('interactive')) {
                    gMacroMgr.appendCommand(command);
                }
            }
            var numRepeats = 1
            // if commands.js is included (e.g. in a dialog) but not isearch.js,
            // then gIsearchController is undefined -- we obviously can't be in
            // a 'repeated command' situation
            if (typeof(gIsearchController) != 'undefined' &&
                gIsearchController.inRepeatCounterAccumulation) {
                numRepeats = gIsearchController.repeatCounter;
                gCancelMultiHandler();
            }
            for (var i = 0; i < numRepeats; i++) {
                if (_commandNode_isCheckCommand(commandNode)) {
                    // For a check command, we do the command, then re-update it,
                    // (as presumably the state has now toggled)
                    // XXX - Can have params here
                    controller.doCommand(command);
                    _command_updateCommand(command, commandNode, controller)
                } else {
                    // XXX - Can have params here
                    controller.doCommand(command);
                }
            }
            rc = true;
        }
        else {
            commandLog.info("Command " + command + " has no controller.");
        }
    }
    catch (e) {
        commandLog.exception(e,"An error occurred executing the "+command+" command");
        throw Components.results.NS_ERROR_FAILURE;
    }
    return rc;
}

// The following controller is for any <command> or <broadcaster>
// that doesn't fit into any other controller.  It is generally
// used for commands that don't ever get disabled.

function command_broadcasterController() {
    this.handlers = {
        'cmd_helpAbout': 'browse_About()',
        'cmd_viewBottomPane': 'uilayout_togglePane(\'bottom_splitter\', \'output_tabs\', \'cmd_viewBottomPane\');',
        'cmd_viewLeftPane': 'uilayout_togglePane(\'workspace_left_splitter\', \'project_toolbox_tabs\', \'cmd_viewLeftPane\');',
        'cmd_viewRightPane': 'uilayout_togglePane(\'workspace_right_splitter\', \'right_toolbox_tabs\', \'cmd_viewRightPane\');',
        'cmd_viewProjects': 'uilayout_toggleTab(\'project_tab\')',
        'cmd_viewToolbox': 'uilayout_toggleTab(\'toolbox_tab\')',
        'cmd_viewCodeBrowser': 'uilayout_toggleTab(\'codebrowser_tab\')',
        'cmd_focusProjectPane': 'uilayout_focusPane(\'project_toolbox_tabs\')',
        'cmd_focusToolboxPane': 'uilayout_focusPane(\'right_toolbox_tabs\')',
        'cmd_focusBottomPane': 'uilayout_focusPane(\'output_tabs\')',
        'cmd_focusEditor': 'gViewMgr.currentView.setFocus()',
        'cmd_focusSource': 'gViewMgr.currentView.viewSource()',
        'cmd_toggleButtonText': 'uilayout_toggleButtons()',
        'cmd_viewedittoolbar': 'uilayout_toggleToolbarVisibility(\'standardToolbar\')',
        'cmd_viewtoolstoolbar': 'uilayout_toggleToolbarVisibility(\'toolsToolbar\')',
        'cmd_viewmacrotoolbar': 'uilayout_toggleToolbarVisibility(\'macrosToolbar\')',
        'cmd_viewkomodotoolbar': 'uilayout_toggleToolbarVisibility(\'komodoToolbar\')',
        'cmd_viewworkspacetoolbar': 'uilayout_toggleToolbarVisibility(\'workspaceToolbar\')',
        'cmd_viewdebugtoolbar': 'uilayout_toggleToolbarVisibility(\'debuggerToolbar\')',
        'cmd_viewfindtoolbar': 'uilayout_toggleToolbarVisibility(\'findToolbar\')',
        'cmd_viewscctoolbar': 'uilayout_toggleToolbarVisibility(\'sccToolbar\')',
        'cmd_viewFullScreen': 'uilayout_FullScreen()',
        'cmd_editPrefs': 'prefs_doGlobalPrefs(null)',
        'cmd_openStartPage': 'open_openStartPage()',
        'cmd_helpHelp': 'launch_MainHelp()',
        'cmd_helpTipOfTheDay': 'launch_TipOfTheDay()',
        'cmd_helpShowKeybindings': 'browse_showKeybindings()',
        'cmd_helpPerlRef_Local': 'browse_PerlRef_Local()',
        'cmd_helpPerlRef_Web': 'browse_PerlRef_Web()',
        'cmd_helpPerlMailingLists': 'browse_PerlMailingLists()',
        'cmd_helpPythonRef_Local': 'browse_PythonRef_Local()',
        'cmd_helpPythonRef_Web': 'browse_PythonRef_Web()',
        'cmd_helpPythonMailingLists': 'browse_PythonMailingLists()',
        'cmd_helpPHPRef_Web': 'browse_PHPRef_Web()',
        'cmd_helpPHPMailingLists': 'browse_PHPMailingLists()',
        'cmd_helpRubyRef_Web': 'browse_RubyRef_Web()',
        'cmd_helpRubyMailingLists': 'browse_RubyMailingLists()',
        'cmd_helpTclRef_Local': 'browse_TclRef_Local()',
        'cmd_helpTclRef_Web': 'browse_TclRef_Web()',
        'cmd_helpTclMailingLists': 'browse_TclMailingLists()',
        'cmd_helpXSLTMailingLists': 'browse_XSLTMailingLists()',
// # if PRODUCT_TYPE == "edit"
        'cmd_tryKomodoIDE': 'browse_tryKomodoIDE()',
// # endif
        'cmd_helpKomodoMailLists': 'browse_MailLists()',
        'cmd_helpActiveState': 'browse_ActiveState()',
        'cmd_helpASPN': 'browse_ASPN()',
        'cmd_helpSupport': 'browse_Support()',
        'cmd_helpUpdates': 'browse_Updates()',
        'cmd_helpViewBugs': 'browse_Bugs()',
        'cmd_helpLanguage': 'launch_LanguageHelp()',
        'cmd_helpLanguageAlternate': 'launch_AlternateLanguageHelp()',
        'cmd_toolsWatchFile': 'launch_watchLocalFile()',
        'cmd_toolsRunCommand': 'launch_RunCommand()',
        'cmd_toolsRx': 'launch_rx()',
        'cmd_open': 'view_openFilesWithPicker()',
        'cmd_open_remote': 'filepicker_openRemoteFiles()',
        'cmd_openTemplate': 'view_openTemplatesWithPicker()',
        'cmd_new': 'gViewMgr.doNewView()',
        'cmd_newTemplate': 'gViewMgr.newTemplate()',
        'cmd_quit': 'KomodoGoQuitApplication()',
        'cmd_findInFiles': 'launch_FindInFiles()',
        'cmd_replaceInFiles': 'launch_ReplaceInFiles()',
        'cmd_httpInspector': 'launch_HttpInspectorWindow()',
        'cmd_nextLintResult': 'lint_jumpToNextLintResult()',
        'cmd_lintClearResults': 'lint_clearResults()',
        'cmd_lintNow': 'lint_doRequest()'
    }
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
command_broadcasterController.prototype = new Controller();
command_broadcasterController.prototype.constructor = command_broadcasterController;

command_broadcasterController.prototype.destructor = function() {
}

command_broadcasterController.prototype.isCommandEnabled = function(cmdName) {
    if (cmdName in this.handlers) {
        return true;
    };
    return false;
}

command_broadcasterController.prototype.supportsCommand = command_broadcasterController.prototype.isCommandEnabled;

command_broadcasterController.prototype.doCommand = function(cmdName) {
    if (cmdName in this.handlers) {
        return eval(this.handlers[cmdName]);
    };
    return false;
}

