/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/*

base Manager

base class for all 'project' viewer manager classes

*/
var gPartManagers = [];

function baseManager(command_name) {
    this.viewMgr = null; // set in subclasses
    this._command_name = null;
    if (typeof(command_name) != 'undefined')
        this._command_name = command_name;
    this.batchUpdate = false;
}

baseManager.prototype.constructor = baseManager;

baseManager.prototype.addItem = function(/* koIPart */ part, index) {
    this.log.error('baseManager.prototype.addItem not implemented');
}

baseManager.prototype.removeItem = function(item, skipdialog) {
    if (typeof(skipdialog)=='undefined') skipdialog = false;
    if (!skipdialog) {
        var container;
        if (this == toolboxMgr) {
            container = 'Toolbox';
        } else if (item.project) {
            container = " project '" + item.project.name + "'";
        } else {
            return false;
        }
        var question = "Do you want to remove the " + item.type +
                       " '" +
                       item.name + "' from the " + container + "?";
        skipdialog = dialog_yesNo(question) == "Yes";
    }
    if (skipdialog) {
        try {
            var parent = item.parent;
            parent.removeChild(item);
            if (!this.batchUpdate)
                this.viewMgr.view.refresh(parent);
        } catch (e) {
            this.log.exception(e);
        }
        return true;
    }
    return false;
}


baseManager.prototype.removeItems = function(items, trash) {
    var sysUtilsSvc = Components.classes["@activestate.com/koSysUtils;1"].
                getService(Components.interfaces.koISysUtils);
    this.batchUpdate = items.length > 1;
    try {
        var item;
        var projects = [];
        for (var i = 0; i < items.length; i++) {
            item = items[i];
            if (item.type == "project") continue;
            // if the parent has been removed already, the item will not be
            // associated with a project.
            if (item.project) {
                if (!(item.project in projects))
                    projects.push(item.project)
                baseManager.prototype.removeItem.apply(this, [item,true])
            }
            if (trash) {
                var file = item.getFile();
                if (file && file.isLocal) {
                    // move to trash
                    sysUtilsSvc.MoveToTrash(file.path);
                }
            }
        }
        if (this.batchUpdate) {
            for (var p=0; p < projects.length; p++) {
                this.viewMgr.view.refresh(projects[p]);
            }
        }
    } catch(e) {
        this.log.exception(e);
    }
    this.batchUpdate = false;
}

baseManager.prototype.renameItem = function(item) {
    var resp = new Object ();
    resp.name= item.name;
    resp.value = item.value;
    resp.res = '';

    if (resp.value) {
        openWindowUniqueInstance("chrome://komodo/content/project/renamePart.xul",
                'komodo_renamePart',
                "chrome,dialog,modal,resizable=yes",
                resp);
        if (resp.res != true) {
            return false;
        }
    } else {
        resp.name = dialog_prompt(
            "Enter the new name.", // prompt
            null, // label
            item.name, // default
            "New Name"); // title
        if (!resp.name) return false;
    }
    /* Modify the project attribute and the wrapped attribute */
    item.name = resp.name;
    return true;
}

baseManager.prototype.getItemsByURL = function(url) {
    this.log.error('baseManager.prototype.getItemsByURL not implemented');
}

baseManager.prototype.refreshView = function() {
    if (this.viewMgr.tree.treeBoxObject) {
        this.viewMgr.tree.treeBoxObject.beginUpdateBatch();
        this.viewMgr.tree.treeBoxObject.invalidate();
        this.viewMgr.tree.treeBoxObject.endUpdateBatch();
    }
}

baseManager.prototype.invalidateItem = function(item) {
    var index = this.viewMgr.view.getIndexByPart(item);
    if (index >= 0) {
        this.viewMgr.tree.treeBoxObject.invalidateRow(index);
        return true;
    }
    return false;
}

baseManager.prototype.findChildByAttributeValue = function (item, attribute, value) {
    try {
        return item.getChildByAttributeValue(attribute, value, true);
    } catch(e) {
        // not a container
        this.log.exception(e);
    }
    return null;
}


baseManager.prototype.isChildOf = function (parent, child) {
    try {
        return parent.hasChild(child);
    } catch(e) {
        this.log.exception(e);
    }
    return false;
}

baseManager.prototype.findItemByURL = function (parent, type, url) {
    var child = parent.getChildWithTypeAndStringAttribute(type, "url", url, true);
    if (child) return child;
    if (parent.url == url) {
        return parent;
    }
    return null;
}

baseManager.prototype.applyPartKeybindings = function () {
    // No default implementation -- up to each partManager to define it,
    // using the callback in the wrappedPart.
}

baseManager.prototype.writeable = function () {
    // No default implementation -- up to each partManager to define it,
}

baseManager.prototype.onDrop = function(event, transferDataSet, session, index) {
    // detect if being draged from a projectviewer, which implies we have a komodo part
    var sourceNode = session.sourceNode;
    var sourceNodeName = null;
    var elementId = null;
    var viewer = null;
    var dragged = null;
    //this.log.setLevel(LOG_DEBUG);
    this.log.debug("session.sourceNode " + session.sourceNode);
    if (session.sourceNode)
        this.log.debug("session.sourceNode.nodeName " + session.sourceNode.nodeName);
    this.log.debug("session.numDropItems " + session.numDropItems);
    if (sourceNode && (sourceNode.localName == 'partviewer' ||
                       sourceNode.localName == 'treechildren')) {
        if (sourceNode.localName == 'treechildren') {
            sourceNode = sourceNode.parentNode.parentNode.parentNode;
        }
        try {
            sourceNodeName = sourceNode.localName;
            this.log.debug("  sourceNodeName = " + sourceNodeName);
            elementId = sourceNode.id;
            this.log.debug("  elementId = " + elementId+'\n');
            viewer = document.getElementById(elementId);
            this.log.debug("  viewer = " + viewer.getAttribute('id'));
            dragged = viewer.getSelectedItems();
            this.log.debug("  dragged = " + dragged);
        } catch(e) {
            this.log.debug(e);
            // it was not a projectviewer
            sourceNodeName = null;
            elementId = null;
            viewer = null;
            dragged = null;
        }
    }
    this.viewMgr.focus();
    var part;

    var droppedon = this.viewMgr.view.getRowItem(index);
    var droppedonIsContainer = this.viewMgr.view.isContainer(index);
    if (!droppedonIsContainer && droppedon) {
        droppedon = droppedon.parent;
    } else if (!droppedon && this != gProjectManager) {
        droppedon = this.toolbox;
        droppedonIsContainer = true;
    }

    // loop through dragged elements
    for ( var i = 0; i < transferDataSet.dataList.length; ++i ) {
        var transferData = transferDataSet.dataList[i];
        // handle the data now
        if (dragged && dragged.length > 0 && sourceNodeName == 'partviewer') {
            // we can only drag from one place at a time, so we should match the returned index!
            var dragpart = dragged[i];

            if (session.dragAction == Components.interfaces.nsIDragService.DRAGDROP_ACTION_COPY) {
                this.log.debug('copying a part of type '+dragpart.type);
                part = dragpart.clone();
                this.addItem(part, droppedon);
            } else {
                this.log.debug('moving a part of type '+dragpart.type);

                // if what we are dropping on is not a container, then make
                // sure that it's parent does not already have
                // the part we're dragging as a direct child
                if (droppedon.hasChild(dragpart)) continue;

                // if we're dropping on a container, make sure
                // that we are not already a child of that
                // container, and that the dropon point is not
                // a child of the drag part
                try {
                    dragpart.QueryInterface(Components.interfaces.koIContainerBase);
                    if (dragpart.hasChild(droppedon)) continue;
                } catch(e) {
                }

                // remove the part from it's old parent if we can add it to the
                // new view.  do not remove children of a live folder, but allow
                // a top level live folder to be moved in a project.
                part = dragpart.clone();
                if (this.addItem(part, droppedon) && !(dragpart.parent.live && dragpart.live))
                    viewer.manager.removeItem(dragpart,true);
            }
        } else {
            // XXX this is hardcoded to file or snippet, should be refactored to allow
            // any extension to examine the dropped data.
            var desc = new Object();
            // from DragDrop.js
            this.log.debug('unpacking the dropped data...');
            unpackDroppedData(transferData.dataList[0], desc)

            //dump ("data = " + desc.text +'\n');
            //logging_dumpObject(desc);
            if (desc.isDebuggerFile) return false;

            if (!desc.isURL) {
                AddSnippetFromText(desc.text,droppedon);
                continue;
            }
            // If we're here, it is a URL
            if (!desc.isFileOrDir) {
                //dump("Adding URL "+desc.text+"\n");
                peURL_addURL(desc.text,droppedon);
                continue;
            }
            // if we're here, it's a URL to something we can open (file or folder)
            //dump("is URL\n");
            if (! desc.isExistingFile) {
                // XXX should do something about "missing files?"
                // for now, fall through.
            }
            //dump("is Existing File\n");

            // handle new lines in droped file uri's
            if (desc.text.search("\n")) {
                var s = desc.text.split("\n");
                desc.name = typeof(s[1])!='undefined'?s[1]:s[0];
                desc.text = s[0];
            }

            if (desc.isDir) {
                // XXX do we want to add a shortcut, or import from filesystem?
                peFile_addDirectoryShortcut(uriparse_URIToLocalPath(desc.text), droppedon);
            } else if (desc.isFileURL && desc.text.match(/\.kpf$/)) {
                project_openProjectFromURL(desc.text);
            } else if (desc.isFileURL && desc.text.match(/\.kpz$/)) {
                part_ImportFromPackage(droppedon, uriparse_URIToLocalPath(desc.text));
            } else {
                // XXX should check that the project doesn't already have the URL
                part = droppedon.project.createPartFromType('file');
                part.setStringAttribute('url', desc.text);
                part.setStringAttribute('name', uriparse_baseName(desc.text));
                this.addItem(part, droppedon);
            }
        }
    } // foreach drag item
    return true;
}

/* looks in all managers for items related to the url so it can update them */
function findItemsByURL(url)
{
    var items = [];
    var list;
    for (var i=0; i < gPartManagers.length; i++) {
        list = gPartManagers[i].getItemsByURL(url);
        items = items.concat(list);
    }
    return items;
}

function removeItemsByURL(url, skipDialog)
{
    var items = [];
    var i, j, list;
    for (i=0; i < gPartManagers.length; i++) {
        list = gPartManagers[i].getItemsByURL(url);
        for (j = 0; j < list.length; j++) {
            gPartManagers[i].removeItem(list[j], skipDialog);
        }
    }
    return items;
}

/* looks in all managers for _parts_ related to the url so it can update them */
function findPartsByURL(url)
{
    var parts = []
    for (var i=0; i < gPartManagers.length; i++) {
        var list = gPartManagers[i].getPartsByURL(url);
        parts = parts.concat(list);
    }
    return parts;
}

function invalidateItem(item)
{
    for (var i=0; i < gPartManagers.length; i++) {
        if (gPartManagers[i].invalidateItem(item)) continue;
    }
}
